'''
Created on Sep 18, 2016

@author: sridharn
'''
import json
import requests
import hashlib
import logging

class VoiceInsights:
    
    appToken = ''
    session = None

        
    def sendVLEvent(self, payload):
        url = "https://api.voicelabs.co/events"
        params = { "auth_token" : payload['app_token']}
        data_json = json.dumps(payload)
        headers = {'Content-type': 'application/json'}
        response = requests.post(url, params=params, data=data_json, headers=headers, timeout=1)
        return response


    def md5(self, input_string):
        return hashlib.md5(input_string).hexdigest()
        
            
    def __init__(self):
        pass
        
    def initialize(self, appToken, session):
        
        try:

            if session is None or appToken is None or len(appToken.strip()) == 0:
                logging.error("ERROR: cannot initialize VoiceInsights SDK. either session or appToken are 'None'")
                return
            
            if session.get('user',{}).has_key('userId') is False:
                logging.error("ERROR: cannot initialize VoiceInsights SDK. User Id is not found in session")
                return
                
            if session.has_key('sessionId') is False:
                logging.error("ERROR: cannot initialize VoiceInsights SDK. sessionId is not found in session")
                return
                
            
            if self.session is not None and self.session['sessionId'] == session['sessionId']:
                # logging.error("WARNING: Redundant Initialization. A session has already been started with sessionId: %s", session['sessionId'])
                return

            self.appToken = appToken
            self.session = session
            p =  {}
            p['app_token'] = self.appToken
            p['user_hashed_id'] = self.md5(self.session['user']['userId'])
            p['session_id'] = self.session['sessionId'];
            p['event_type'] = 'INITIALIZE'
            p['data'] = None
            resp = self.sendVLEvent(p)
            return resp

        except:
            logging.error("ERROR: occurred inside initalize")
            return None

    def track(self, intent_name, intent_request, response):
    
        try:    
            if self.session is None:
                logging.error("ERROR: Voice Insights has not been initialized. Initalize() method need to have been invoked before tracking")
                return
                  
            p =  {}
            p['app_token'] = self.appToken
            p['user_hashed_id'] = self.md5(self.session['user']['userId'])
            p['session_id'] = self.session['sessionId'];
            p['event_type'] = 'SPEECH'
            p['intent'] = intent_name
            p['data'] = {}
            
            if intent_request is not None and intent_request.get('intent',{}).has_key('slots'):
                p['data']['metadata'] = intent_request['intent']['slots']
            else:
                p['data']['metadata'] = None
            
            if isinstance(response, basestring):
                p['data']['speech'] = response

            elif response is not None and response.has_key('response') and response.get('response', {}).has_key('outputSpeech') is True:

                speechObj = response['response']['outputSpeech']

                if speechObj.has_key('type'):
                    
                    if speechObj['type'] == 'SSML':
                        p['data']['speech'] = response['response']['outputSpeech']['ssml']

                    elif speechObj['type'] == 'PlainText':
                        p['data']['speech'] = response['response']['outputSpeech']['text']

                    else:
                        logging.error("ERROR: passed a response object with an unknown Type") 

                else:
                    logging.error("ERROR: passed a response object thats not an Alexa response") 

            else:
                p['data']['speech'] = None

            resp = self.sendVLEvent(p)             
            return resp

        except:
            logging.error("ERROR: occurred inside track method")
            return None
    



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='voicelabs',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.0.9',

    description='VoiceInsights Python SDK',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/voicelabs/analytics-python-sdk',

    # Author details
    author='Sridhar Nallani',
    author_email='sridhar@voicelabs.co',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        #'Development Status :: 1 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7',
    ],

    # What does your project relate to?
    keywords='voicelabs, voiceinsights, alexasdk',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=['voicelabs'],
    #find_packages(exclude=['contrib', 'docs', 'tests']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['requests'],


)
